--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--								University of Hawaii at Manoa						         --
--						Instrumentation Development Lab / GARY S. VARNER				--
--   								Watanabe Hall Room 214								      --
--  								  2505 Correa Road										   --
--  								 Honolulu, HI 96822											--
--  								Lab: (808) 956-2920											--
--	 								Fax: (808) 956-2930										   --
--  						E-mail: idlab@phys.hawaii.edu									   --
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------	
-- Design by: Larry L. Ruckman Jr.															--
-- DATE : 26 JUNE 2007																			--
-- Project name: ICRR firmware																--
--	Module name: GND_16			  																--
--	Description : 																					--
-- 	GND_BUS 16 bits wide																		--
--		  											    												--
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity FO_USB_MUX is
   port ( 
			 xWAKEUP	  			: in    std_logic;
			 xUSB_CLR_ALL		: in    std_logic;
			 xFO_CLR_ALL		: in    std_logic;
			 xUSB_REFRESH	 	: in    std_logic;
			 xFO_REFRESH	 	: in    std_logic;
			 xUSB_PED_COL		: in    std_logic_vector (4 downto 0);
			 xFO_PED_COL	 	: in    std_logic_vector (4 downto 0);
			 xUSB_PED_ROW		: in    std_logic_vector (2 downto 0);
			 xFO_PED_ROW	 	: in    std_logic_vector (2 downto 0);
			 xUSB_THRES0		: in    std_logic_vector (15 downto 0);
			 xFO_THRES0		 	: in    std_logic_vector (15 downto 0);
			 xUSB_THRES1		: in    std_logic_vector (15 downto 0);
			 xFO_THRES1		 	: in    std_logic_vector (15 downto 0);
			 xUSB_THRES2		: in    std_logic_vector (15 downto 0);
			 xFO_THRES2		 	: in    std_logic_vector (15 downto 0);
			 xUSB_THRES3		: in    std_logic_vector (15 downto 0);
			 xFO_THRES3		 	: in    std_logic_vector (15 downto 0);
			 xUSB_WBIAS			: in    std_logic_vector (11 downto 0);
			 xFO_WBIAS		 	: in    std_logic_vector (11 downto 0);
			 xUSB_TERM			: in    std_logic_vector (2 downto 0);
			 xFO_TERM		 	: in    std_logic_vector (2 downto 0);
			 xUSB_SGN			: in    std_logic;
			 xFO_SGN				: in    std_logic;
			 xUSB_EN_PED		: in    std_logic;
			 xFO_EN_PED			: in    std_logic;
			 xUSB_RSTRB			: in    std_logic;
			 xFO_RSTRB	  		: in    std_logic;
			 xUSB_DONE		 	: in    std_logic;
			 xFO_DONE	 		: in    std_logic;
			 xUSB_SOFT_TRIG	: in    std_logic;
			 xFO_SOFT_TRIG		: in    std_logic;
			 xUSB_RADDR			: in    std_logic_vector (15 downto 0);
			 xFO_RADDR			: in    std_logic_vector (15 downto 0);			 
			 xFO_LOSS			: in    std_logic;
			 xUSB_RSEL			: in    std_logic_vector (3 downto 0);
			 xFO_RSEL	 		: in    std_logic_vector (3 downto 0);
			 xPED_COL_ADDR 	: out   std_logic_vector (4 downto 0);
			 xPED_ROW_ADDR 	: out   std_logic_vector (2 downto 0);
			 xTHRES0	      	: out   std_logic_vector (15 downto 0);
			 xTHRES1	     		: out   std_logic_vector (15 downto 0);
			 xTHRES2	      	: out   std_logic_vector (15 downto 0);
			 xTHRES3	      	: out   std_logic_vector (15 downto 0);
			 xWBIAS	      	: out   std_logic_vector (11 downto 0);
			 xTERM	    	   : out   std_logic_vector (2 downto 0);
			 xSGN					: out   std_logic;	
          xEN_PED   			: out   std_logic;
			 xRSEL				: out   std_logic_vector (3 downto 0);	
			 xRX_LOST			: out   std_logic;				 
			 xRSTRB				: out   std_logic;
			 xDONE				: out   std_logic;
			 xCLR_ALL			: out   std_logic;
			 xREFRESH			: out   std_logic;
			 xSOFT_TRIG			: out   std_logic;
			 xRADDR				: out   std_logic_vector (15 downto 0));
end FO_USB_MUX;

architecture Behavioral of FO_USB_MUX is
	signal RADDR 			: std_logic_vector(15 downto 0);
	signal RX_LOST 		: std_logic;
	signal DONE 			: std_logic;
	signal CLR_ALL 		: std_logic;
	signal RSTRB 			: std_logic;
	signal SOFT_TRIG 		: std_logic;
	signal REFRESH			: std_logic;
	signal PED_COL_ADDR	: std_logic_vector(4 downto 0);
	signal PED_ROW_ADDR	: std_logic_vector(2 downto 0);
	signal THRES0 			: std_logic_vector(15 downto 0);
	signal THRES1 			: std_logic_vector(15 downto 0);
	signal THRES2 			: std_logic_vector(15 downto 0);
	signal THRES3 			: std_logic_vector(15 downto 0);
	signal WBIAS 			: std_logic_vector(11 downto 0);
	signal TERM 			: std_logic_vector(2 downto 0);
	signal SGN				: std_logic;
	signal EN_PED			: std_logic;
	signal RSEL				: std_logic_vector(3 downto 0);
--------------------------------------------------------------------------------
--   								components     		   						         --
--------------------------------------------------------------------------------
   component BUF
      port ( I : in    std_logic; 
             O : out   std_logic);
   end component;
--------------------------------------------------------------------------------	
	component BUF16
    Port ( 
			  I   : in   std_logic_vector (15 downto 0);
			  O	: out  std_logic_vector (15 downto 0));
   end component;
--------------------------------------------------------------------------------	
	component BUF12
    Port ( 
			  I   : in   std_logic_vector (11 downto 0);
			  O	: out  std_logic_vector (11 downto 0));
   end component;
--------------------------------------------------------------------------------	
	component BUF5
    Port ( 
			  I   : in   std_logic_vector (4 downto 0);
			  O	: out  std_logic_vector (4 downto 0));
   end component;
--------------------------------------------------------------------------------	
	component BUF4
    Port ( 
			  I   : in   std_logic_vector (3 downto 0);
			  O	: out  std_logic_vector (3 downto 0));
   end component;
--------------------------------------------------------------------------------	
	component BUF3
    Port ( 
			  I   : in   std_logic_vector (2 downto 0);
			  O	: out  std_logic_vector (2 downto 0));
   end component;
--------------------------------------------------------------------------------
begin
--------------------------------------------------------------------------------
	process(xWAKEUP)
	begin
		if xWAKEUP = '1' then
			RX_LOST 			<= '1';	 
			RADDR 			<= xUSB_RADDR;
			RSTRB 			<= xUSB_RSTRB;	 
			DONE 				<= xUSB_DONE;	 
			CLR_ALL 			<= xUSB_CLR_ALL;	 
			SOFT_TRIG 		<= xUSB_SOFT_TRIG;
			REFRESH 			<= xUSB_REFRESH;	
			PED_ROW_ADDR	<= xUSB_PED_ROW;				
			PED_COL_ADDR	<= xUSB_PED_COL;				
			THRES0 			<= xUSB_THRES0;						
			THRES1 			<= xUSB_THRES1;	
			THRES2 			<= xUSB_THRES2;	
			THRES3 			<= xUSB_THRES3;	
			WBIAS 			<= xUSB_WBIAS;	
			TERM	 			<= xUSB_TERM;	
			SGN	 			<= xUSB_SGN;	
			EN_PED 			<= xUSB_EN_PED;	
			RSEL				<= xUSB_RSEL;
		else
			RX_LOST 			<= xFO_LOSS;	 
			RADDR 			<= xFO_RADDR;
			RSTRB 			<= xFO_RSTRB;	 
			DONE 				<= xFO_DONE;	 
			CLR_ALL 			<= xFO_CLR_ALL;	 
			SOFT_TRIG 		<= xFO_SOFT_TRIG;
			REFRESH 			<= xFO_REFRESH;	
			PED_ROW_ADDR	<= xFO_PED_ROW;				
			PED_COL_ADDR	<= xFO_PED_COL;				
			THRES0 			<= xFO_THRES0;						
			THRES1 			<= xFO_THRES1;	
			THRES2 			<= xFO_THRES2;	
			THRES3 			<= xFO_THRES3;	
			WBIAS 			<= xFO_WBIAS;	
			TERM	 			<= xFO_TERM;	
			SGN	 			<= xFO_SGN;	
			EN_PED 			<= xFO_EN_PED;	
			RSEL				<= xFO_RSEL;
		end if;
	end process;
--------------------------------------------------------------------------------
	xBUF_RX_LOST : BUF 
	port map (
		I	=> RX_LOST,
		O	=> xRX_LOST);
--------------------------------------------------------------------------------
	xBUF_RADDR : BUF16
	port map (
		I	=> RADDR,
		O	=> xRADDR);
--------------------------------------------------------------------------------
	xBUF_RSTRB : BUF 
	port map (
		I	=> RSTRB,
		O	=> xRSTRB);
--------------------------------------------------------------------------------
	xBUF_DONE : BUF 
	port map (
		I	=> DONE,
		O	=> xDONE);
--------------------------------------------------------------------------------
	xBUF_CLR_ALL : BUF 
	port map (
		I	=> CLR_ALL,
		O	=> xCLR_ALL);
--------------------------------------------------------------------------------
	xBUF_SOFT_TRIG : BUF 
	port map (
		I	=> SOFT_TRIG,
		O	=> xSOFT_TRIG);
--------------------------------------------------------------------------------
	xBUF_REFRESH : BUF 
	port map (
		I	=> REFRESH,
		O	=> xREFRESH);
--------------------------------------------------------------------------------
	xBUF_PED_ROW_ADDR : BUF3 
	port map (
		I	=> PED_ROW_ADDR,
		O	=> xPED_ROW_ADDR);
--------------------------------------------------------------------------------
	xBUF_PED_COL_ADDR : BUF5 
	port map (
		I	=> PED_COL_ADDR,
		O	=> xPED_COL_ADDR);
--------------------------------------------------------------------------------
	xBUF_THRES0 : BUF16 
	port map (
		I	=> THRES0,
		O	=> xTHRES0);
--------------------------------------------------------------------------------
	xBUF_THRES1 : BUF16 
	port map (
		I	=> THRES1,
		O	=> xTHRES1);
--------------------------------------------------------------------------------
	xBUF_THRES2 : BUF16 
	port map (
		I	=> THRES2,
		O	=> xTHRES2);
--------------------------------------------------------------------------------
	xBUF_THRES3 : BUF16 
	port map (
		I	=> THRES3,
		O	=> xTHRES3);
--------------------------------------------------------------------------------
	xBUF_EN_PED : BUF 
	port map (
		I	=> EN_PED,
		O	=> xEN_PED);
--------------------------------------------------------------------------------
	xBUF_WBIAS : BUF12 
	port map (
		I  => WBIAS,
		O  => xWBIAS);
--------------------------------------------------------------------------------
	xBUF_TERM : BUF3 
	port map (
		I  => TERM,
		O  => xTERM);
--------------------------------------------------------------------------------
	xBUF_SGN : BUF 
	port map (
		I  => SGN,
		O  => xSGN);
--------------------------------------------------------------------------------
	xBUF_RSEL : BUF4 
	port map (
		I	=> RSEL,
		O	=> xRSEL);
--------------------------------------------------------------------------------
end Behavioral;