--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--								University of Hawaii at Manoa						         --
--						Instrumentation Development Lab / GARY S. VARNER				--
--   								Watanabe Hall Room 214								      --
--  								  2505 Correa Road										   --
--  								 Honolulu, HI 96822											--
--  								Lab: (808) 956-2920											--
--	 								Fax: (808) 956-2930										   --
--  						E-mail: idlab@phys.hawaii.edu									   --
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------	
-- Design by: Larry L. Ruckman Jr.															--
-- DATE : 27 JUNE 2007																			--
-- Project name: ICRR firmware																--
--	Module name: HK_CLK			  																--
--	Description : 																					--
-- 	House Keeping clock control module													--
--		  											    												--
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity HK_CLK is
    Port ( 	xCLK_15MHz	: 	in std_logic;
				xCLR_ALL		: 	in std_logic;
				xCLK_1Hz 	: 	out std_logic;
				xCLK_10Hz 	: 	out std_logic;
				xCLK_100Hz 	: 	out std_logic;
				xCLK_1kHz 	: 	out std_logic);
end HK_CLK;

architecture Behavioral of HK_CLK is
	type STATE_TYPE is (	HI_1Hz, 		LO_1Hz, 
								HI_10Hz, 	LO_10Hz, 
								HI_100Hz, 	LO_100Hz, 
								HI_1kHz, 	LO_1kHz);								
	signal STATE_1Hz  	: STATE_TYPE;
	signal STATE_10Hz  	: STATE_TYPE;
	signal STATE_100Hz  	: STATE_TYPE;
	signal STATE_1kHz  	: STATE_TYPE;
	signal CLK_1Hz			: std_logic;
	signal CLK_10Hz		: std_logic;
	signal CLK_100Hz		: std_logic;
	signal CLK_1kHz		: std_logic;
	signal CNT_1Hz			: std_logic_vector(23 downto 0);	
	signal CNT_10Hz		: std_logic_vector(23 downto 0);	
	signal CNT_100Hz		: std_logic_vector(23 downto 0);	
	signal CNT_1kHz		: std_logic_vector(23 downto 0);	
--------------------------------------------------------------------------------
--   								components     		   						         --
--------------------------------------------------------------------------------
   component BUF
      port ( I : in    std_logic; 
             O : out   std_logic);
   end component;
--------------------------------------------------------------------------------	
begin 
--------------------------------------------------------------------------------
	process(xCLK_15MHz,xCLR_ALL)
	begin
		if xCLR_ALL = '1' then
			CNT_1kHz <= (others=>'0');
		elsif rising_edge(xCLK_15MHz) then
--------------------------------------------------------------------------------			
			case STATE_1kHz is
--------------------------------------------------------------------------------	
				when HI_1kHz =>			
					CLK_1kHz <= '1';
					CNT_1kHz <= CNT_1kHz + 1;
					if CNT_1kHz = 7500 then
						CNT_1kHz <= (others=>'0');
						STATE_1kHz <= LO_1kHz;
					end if;
--------------------------------------------------------------------------------	
				when LO_1kHz =>			
					CLK_1kHz <= '0';
					CNT_1kHz <= CNT_1kHz + 1;
					if CNT_1kHz = 7500 then
						CNT_1kHz <= (others=>'0');
						STATE_1kHz <= HI_1kHz;
					end if;
--------------------------------------------------------------------------------	
				when others =>	STATE_1kHz<=HI_1kHz;																
			end case;
		end if;
	end process;	
--------------------------------------------------------------------------------	
	process(xCLK_15MHz,xCLR_ALL)
	begin
		if xCLR_ALL = '1' then
			CNT_100Hz <= (others=>'0');
		elsif rising_edge(xCLK_15MHz) then
--------------------------------------------------------------------------------			
			case STATE_100Hz is
--------------------------------------------------------------------------------	
				when HI_100Hz =>			
					CLK_100Hz <= '1';
					CNT_100Hz <= CNT_100Hz + 1;
					if CNT_100Hz = 75000 then
						CNT_100Hz <= (others=>'0');
						STATE_100Hz <= LO_100Hz;
					end if;
--------------------------------------------------------------------------------	
				when LO_100Hz =>			
					CLK_100Hz <= '0';
					CNT_100Hz <= CNT_100Hz + 1;
					if CNT_100Hz = 75000 then
						CNT_100Hz <= (others=>'0');
						STATE_100Hz <= HI_100Hz;
					end if;
--------------------------------------------------------------------------------	
				when others =>	STATE_100Hz<=HI_100Hz;																
			end case;
		end if;
	end process;	
--------------------------------------------------------------------------------	
	process(xCLK_15MHz,xCLR_ALL)
	begin
		if xCLR_ALL = '1' then
			CNT_10Hz <= (others=>'0');
		elsif rising_edge(xCLK_15MHz) then
--------------------------------------------------------------------------------			
			case STATE_10Hz is
--------------------------------------------------------------------------------	
				when HI_10Hz =>			
					CLK_10Hz <= '1';
					CNT_10Hz <= CNT_10Hz + 1;
					if CNT_10Hz = 750000 then
						CNT_10Hz <= (others=>'0');
						STATE_10Hz <= LO_10Hz;
					end if;
--------------------------------------------------------------------------------	
				when LO_10Hz =>			
					CLK_10Hz <= '0';
					CNT_10Hz <= CNT_10Hz + 1;
					if CNT_10Hz = 750000 then
						CNT_10Hz <= (others=>'0');
						STATE_10Hz <= HI_10Hz;
					end if;
--------------------------------------------------------------------------------	
				when others =>	STATE_10Hz<=HI_10Hz;																
			end case;
		end if;
	end process;	
--------------------------------------------------------------------------------	
	process(xCLK_15MHz,xCLR_ALl)
	begin
		if xCLR_ALL = '1' then
			CNT_1Hz <= (others=>'0');
		elsif rising_edge(xCLK_15MHz) then
--------------------------------------------------------------------------------			
			case STATE_1Hz is
--------------------------------------------------------------------------------	
				when HI_1Hz =>			
					CLK_1Hz <= '1';
					CNT_1Hz <= CNT_1Hz + 1;
					if CNT_1Hz = 7500000 then
						CNT_1Hz <= (others=>'0');
						STATE_1Hz <= LO_1Hz;
					end if;
--------------------------------------------------------------------------------	
				when LO_1Hz =>			
					CLK_1Hz <= '0';
					CNT_1Hz <= CNT_1Hz + 1;
					if CNT_1Hz = 7500000 then
						CNT_1Hz <= (others=>'0');
						STATE_1Hz <= HI_1Hz;
					end if;
--------------------------------------------------------------------------------	
				when others =>	STATE_1Hz<=HI_1Hz;																
			end case;
		end if;
	end process;	
--------------------------------------------------------------------------------	
	xBUF_CLK_1Hz : buf 
	port map (
		I => CLK_1Hz,
		O => xCLK_1Hz);		
--------------------------------------------------------------------------------
	xBUF_CLK_10Hz : buf 
	port map (
		I => CLK_10Hz,
		O => xCLK_10Hz);		
--------------------------------------------------------------------------------
	xBUF_CLK_100Hz : buf 
	port map (
		I => CLK_100Hz,
		O => xCLK_100Hz);		
--------------------------------------------------------------------------------
	xBUF_CLK_1kHz : buf 
	port map (
		I => CLK_1kHz,
		O => xCLK_1kHz);		
--------------------------------------------------------------------------------
end Behavioral;