--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--								University of Hawaii at Manoa						         --
--						Instrumentation Development Lab / GARY S. VARNER				--
--   								Watanabe Hall Room 214								      --
--  								  2505 Correa Road										   --
--  								 Honolulu, HI 96822											--
--  								Lab: (808) 956-2920											--
--	 								Fax: (808) 956-2930										   --
--  						E-mail: idlab@phys.hawaii.edu			 						   --
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------	
-- Design by: Larry L. Ruckman Jr.															--
-- DATE : 29 JUNE 2007																			--
-- Project name: ROBUSTv3 firmware															--
--	Module name: TRIG_DAC		  																--
--	Description : 																					--
-- 	Top Level for Trigger Dac Control													--
--		  											    												--
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity SLAVE_REG is
   port ( xCLK		     	: in  std_logic;
			 xREFRESH     	: in  std_logic;
			 xPED_COL_ADDR : in  std_logic_vector (4 downto 0);
			 xPED_ROW_ADDR : in  std_logic_vector (2 downto 0);
			 xTHRES0	      : in  std_logic_vector (15 downto 0);
			 xTHRES1	      : in  std_logic_vector (15 downto 0);
			 xTHRES2	      : in  std_logic_vector (15 downto 0);
			 xTHRES3	      : in  std_logic_vector (15 downto 0);
			 xWBIAS	      : in  std_logic_vector (11 downto 0);
			 xTERM	      : in  std_logic_vector (2 downto 0);
          xEN_PED   		: in  std_logic;
          xSGN   			: in  std_logic;
			 xMAIN_RBUSY	: in  std_logic;
          xCLR_ALL  		: in  std_logic;
			 xWSTRB			: out std_logic;
			 xSLAVE_WBUSY	: out std_logic;
          xWSEL    		: out std_logic_vector (3 downto 0);
			 xMAIN_DATA	   : out std_logic_vector (15 downto 0));
end SLAVE_REG;

architecture Behavioral of SLAVE_REG is
	type STATE_TYPE is ( SETUP, BREAK_DOWN,
								LOAD_LoCMD, LATCH_LoCMD,
								LOAD_HiCMD, LATCH_HiCMD,
								LOAD_CMD, LATCH_CMD); 
	signal STATE			: STATE_TYPE;
	signal LoCMD 			: std_logic_vector(15 downto 0);
	signal HiCMD 			: std_logic_vector(15 downto 0);
	signal MAIN_DATA 		: std_logic_vector(15 downto 0);
	signal SEL				: std_logic_vector(3 downto 0);
	signal SEL_LUT			: std_logic_vector(3 downto 0);
	signal PHASE_CNT		: std_logic_vector(3 downto 0);
	signal WSTRB 			: std_logic;
	signal SLAVE_WBUSY	: std_logic;
	signal CLR_UPDATE		: std_logic;
	signal UPDATE			: std_logic;
--------------------------------------------------------------------------------
--   								components     		   						         --
--------------------------------------------------------------------------------
   component BUF
      port ( I : in    std_logic; 
             O : out   std_logic);
   end component;
--------------------------------------------------------------------------------	
	component BUF16
    Port ( 
			  I   : in   std_logic_vector (15 downto 0);
			  O	: out  std_logic_vector (15 downto 0));
   end component;
--------------------------------------------------------------------------------	
	component BUF4
    Port ( 
			  I   : in   std_logic_vector (3 downto 0);
			  O	: out  std_logic_vector (3 downto 0));
   end component;
--------------------------------------------------------------------------------
begin
--------------------------------------------------------------------------------
	xBUF_MAIN_DATA : BUF16 
	port map (
		I	=> MAIN_DATA,
		O	=>	xMAIN_DATA);
--------------------------------------------------------------------------------
	xBUF_SEL : BUF4 
	port map (
		I	=> SEL,
		O	=>	xWSEL);
--------------------------------------------------------------------------------
	xBUF_WSTRB : BUF 
	port map (
		I	=> WSTRB,
		O	=>	xWSTRB);
--------------------------------------------------------------------------------
	xBUF_SLAVE_WBUSY : BUF 
	port map (
		I	=> SLAVE_WBUSY,
		O	=>	xSLAVE_WBUSY);
--------------------------------------------------------------------------------
	process(xREFRESH,CLR_UPDATE,xCLR_ALL)
	begin	
		if xCLR_ALL = '1' or CLR_UPDATE = '1' then
			UPDATE <= '0';
		elsif falling_edge(xREFRESH) then
			UPDATE <= '1';	
		end if;
	end process;
--------------------------------------------------------------------------------
	process(xCLR_ALL,xCLK,UPDATE)
	begin
		if xCLR_ALL = '1' then
			CLR_UPDATE 	<= '0';
			MAIN_DATA 	<= x"0000";
			SEL 			<= "0000";
			WSTRB			<= '0';
			SLAVE_WBUSY <= '0';
			PHASE_CNT 	<= (others=>'0');
			STATE			<= SETUP;
		elsif rising_edge(xCLK) and xMAIN_RBUSY = '0' then
			CLR_UPDATE 	<= '0';
--------------------------------------------------------------------------------			
			case STATE is
--------------------------------------------------------------------------------	
				when SETUP =>	
					if UPDATE = '1' then
						SLAVE_WBUSY <= '1';
						SEL 			<= SEL_LUT;
						STATE			<= LOAD_LoCMD;
					end if;
--------------------------------------------------------------------------------	
				when LOAD_LoCMD =>	
					MAIN_DATA 	<= LoCMD;
					WSTRB			<= '0';
					STATE			<= LATCH_LoCMD;
--------------------------------------------------------------------------------	
				when LATCH_LoCMD =>	
					WSTRB			<= '1';
					STATE			<= LOAD_HiCMD;
--------------------------------------------------------------------------------	
				when LOAD_HiCMD =>	
					MAIN_DATA 	<= HiCMD;
					WSTRB			<= '0';
					STATE			<= LATCH_HiCMD;
--------------------------------------------------------------------------------	
				when LATCH_HiCMD =>	
					WSTRB			<= '1';
					STATE			<= LOAD_CMD;
--------------------------------------------------------------------------------	
				when LOAD_CMD =>	
					WSTRB			<= '0';
					STATE			<= LATCH_CMD;
--------------------------------------------------------------------------------	
				when LATCH_CMD =>	
					WSTRB			<= '1';
					PHASE_CNT 	<= PHASE_CNT + 1;
					STATE			<= BREAK_DOWN;
--------------------------------------------------------------------------------	
				when BREAK_DOWN =>	
					SEL 			<= "0000";
					WSTRB			<= '0';
					if PHASE_CNT = 9 then
						SLAVE_WBUSY <= '0';
						PHASE_CNT 	<= (others=>'0');
						CLR_UPDATE 	<= '1';
					end if;
					STATE			<= SETUP;
--------------------------------------------------------------------------------	
				when others =>	STATE<=SETUP;																
			end case;
		end if;
	end process;	
--------------------------------------------------------------------------------	
	process(PHASE_CNT)
	begin
		if PHASE_CNT = 0 then
			SEL_LUT	<= "0001";
			LoCMD 	<= x"0003";
			HiCMD 	<= x"0" & xTHRES0(11 downto 0);
		elsif PHASE_CNT = 1 then
			SEL_LUT	<= "0010";
			LoCMD 	<= x"0003";
			HiCMD 	<= x"0" & xTHRES1(11 downto 0);
		elsif PHASE_CNT = 2 then
			SEL_LUT	<= "0100";
			LoCMD 	<= x"0003";
			HiCMD 	<= x"0" & xTHRES2(11 downto 0);
		elsif PHASE_CNT = 3 then
			SEL_LUT	<= "1000";
			LoCMD 	<= x"0003";
			HiCMD 	<= x"0" & xTHRES3(11 downto 0);
		elsif PHASE_CNT = 4 then
			SEL_LUT	<= "1111";
			LoCMD 	<= x"0004";
			HiCMD 	<= x"000" & "000" & xEN_PED;
		elsif PHASE_CNT = 5 then 
			SEL_LUT	<= "1111";
			LoCMD 	<= x"0005";
			HiCMD 	<= x"0" & xPED_ROW_ADDR & xPED_COL_ADDR & x"0";
		elsif PHASE_CNT = 6 then
			SEL_LUT	<= "1111";
			LoCMD 	<= x"0006";
			HiCMD 	<= x"0" & xWBIAS(11 downto 0);
		elsif PHASE_CNT = 7 then
			SEL_LUT	<= "1111";
			LoCMD 	<= x"0007";
			HiCMD 	<= x"000" & "000" & xSGN;
		elsif PHASE_CNT = 8 then
			SEL_LUT	<= "1111";
			LoCMD 	<= x"0008";
			HiCMD 	<= x"000" & '0' & xTERM(2 downto 0);
		else
			SEL_LUT	<= x"0";
			LoCMD 	<= x"0000";
			HiCMD 	<= x"0000";
		end if;
	end process;
--------------------------------------------------------------------------------
end Behavioral;