--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--								University of Hawaii at Manoa						         --
--						Instrumentation Development Lab / GARY S. VARNER				--
--   								Watanabe Hall Room 214								      --
--  								  2505 Correa Road										   --
--  								 Honolulu, HI 96822											--
--  								Lab: (808) 956-2920											--
--	 								Fax: (808) 956-2930										   --
--  						E-mail: idlab@phys.hawaii.edu									   --
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------	
-- Design by: Larry L. Ruckman Jr.															--
-- DATE : 16 JUNE 2007																			--
-- Project name: ICRR firmware																--
--	Module name: USBread  																		--
--	Description : 																					--
-- 	USB 2.0 reading data from PC module													--
--		  											    												--
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity USBread is
   port ( xIFCLK     	: in    std_logic;
          xUSB_DATA  	: in    std_logic_vector (15 downto 0);
          xFLAGA    		: in    std_logic;
			 xRESET    		: in    std_logic;
          xWBUSY    		: in    std_logic;
			 xREFRESH		: out   std_logic;
			 xPED_COL_ADDR : out   std_logic_vector (4 downto 0);
			 xPED_ROW_ADDR : out   std_logic_vector (2 downto 0);
			 xTHRES0	      : out   std_logic_vector (15 downto 0);
			 xTHRES1	      : out   std_logic_vector (15 downto 0);
			 xTHRES2	      : out   std_logic_vector (15 downto 0);
			 xTHRES3	      : out   std_logic_vector (15 downto 0);
			 xWBIAS	      : out   std_logic_vector (11 downto 0);
			 xTERM	      : out   std_logic_vector (2 downto 0);
			 xSGN				: out   std_logic;			 
          xEN_PED   		: out   std_logic;
          xFIFOADR  		: out   std_logic_vector (1 downto 0);
          xRBUSY    		: out   std_logic;
          xSLOE     		: out   std_logic;
          xSLRD     		: out   std_logic;
          xSYNC_USB 		: out   std_logic;
          xSOFT_TRIG		: out   std_logic;
          xTOGGLE   		: out   std_logic);
end USBread;

architecture BEHAVIORAL of USBread is
	type State_type is(st1_WAIT,
							st1_READ, st2_READ, st3_READ,st4_READ,
							st1_SAVE, st1_TARGET, ENDDELAY);
	signal state: State_type;
	signal dbuffer			: std_logic_vector (15 downto 0);
	signal Locmd			: std_logic_vector (15 downto 0);
	signal Hicmd			: std_logic_vector (15 downto 0);
	signal again			: std_logic_vector (1 downto 0);
	signal TOGGLE			: std_logic;
	signal SOFT_TRIG		: std_logic;
	signal SYNC_USB		: std_logic;
	signal SLRD				: std_logic;
	signal SLOE				: std_logic;
	signal RBUSY			: std_logic;
	signal FIFOADR    	: std_logic_vector (1 downto 0);
	signal REFRESH			: std_logic;
	signal PED_COL_ADDR	: std_logic_vector(4 downto 0);
	signal PED_ROW_ADDR	: std_logic_vector(2 downto 0);
	signal THRES0 			: std_logic_vector(15 downto 0);
	signal THRES1 			: std_logic_vector(15 downto 0);
	signal THRES2 			: std_logic_vector(15 downto 0);
	signal THRES3 			: std_logic_vector(15 downto 0);
	signal WBIAS 			: std_logic_vector(11 downto 0);
	signal TERM 			: std_logic_vector(2 downto 0);
	signal SGN				: std_logic;
	signal EN_PED			: std_logic;
--------------------------------------------------------------------------------
--   								components     		   						         --
--------------------------------------------------------------------------------
   component BUF
      port ( I : in    std_logic; 
             O : out   std_logic);
   end component;
--------------------------------------------------------------------------------	
	component BUF16
    Port ( 
			  I   : in   std_logic_vector (15 downto 0);
			  O	: out  std_logic_vector (15 downto 0));
   end component;
--------------------------------------------------------------------------------	
	component BUF12
    Port ( 
			  I   : in   std_logic_vector (11 downto 0);
			  O	: out  std_logic_vector (11 downto 0));
   end component;
--------------------------------------------------------------------------------	
	component BUF3
    Port ( 
			  I   : in   std_logic_vector (2 downto 0);
			  O	: out  std_logic_vector (2 downto 0));
   end component;
--------------------------------------------------------------------------------	
	component BUF5
    Port ( 
			  I   : in   std_logic_vector (4 downto 0);
			  O	: out  std_logic_vector (4 downto 0));
   end component;
--------------------------------------------------------------------------------	
	component BUF32
    Port ( 
			  I   : in   std_logic_vector (31 downto 0);
			  O	: out  std_logic_vector (31 downto 0));
   end component;
--------------------------------------------------------------------------------
begin
--------------------------------------------------------------------------------
	xBUF_REFRESH : BUF 
	port map (
		I  => REFRESH,
		O  => xREFRESH);
--------------------------------------------------------------------------------
	xBUF_PED_ROW_ADDR : BUF3 
	port map (
		I  => PED_ROW_ADDR,
		O  => xPED_ROW_ADDR);
--------------------------------------------------------------------------------
	xBUF_PED_COL_ADDR : BUF5 
	port map (
		I  => PED_COL_ADDR,
		O  => xPED_COL_ADDR);
--------------------------------------------------------------------------------
	xBUF_THRES0 : BUF16 
	port map (
		I  => THRES0,
		O  => xTHRES0);
--------------------------------------------------------------------------------
	xBUF_THRES1 : BUF16 
	port map (
		I  => THRES1,
		O  => xTHRES1);
--------------------------------------------------------------------------------
	xBUF_THRES2 : BUF16 
	port map (
		I  => THRES2,
		O  => xTHRES2);
--------------------------------------------------------------------------------
	xBUF_THRES3 : BUF16 
	port map (
		I  => THRES3,
		O  => xTHRES3);
--------------------------------------------------------------------------------
	xBUF_WBIAS : BUF12 
	port map (
		I  => WBIAS,
		O  => xWBIAS);
--------------------------------------------------------------------------------
	xBUF_TERM : BUF3 
	port map (
		I  => TERM,
		O  => xTERM);
--------------------------------------------------------------------------------
	xBUF_SGN : BUF 
	port map (
		I  => SGN,
		O  => xSGN);
--------------------------------------------------------------------------------
	xBUF_EN_PED : BUF 
	port map (
		I  => EN_PED,
		O  => xEN_PED);
--------------------------------------------------------------------------------
	xBUF_TOGGLE : BUF 
	port map (
		I  => TOGGLE,
		O  => xTOGGLE);
--------------------------------------------------------------------------------		
	xBUF_SOFT_TRIG : BUF 
	port map (
		I  => SOFT_TRIG,
		O  => xSOFT_TRIG);
--------------------------------------------------------------------------------		
	xBUF_SYNC_USB : BUF 
	port map (
		I  => SYNC_USB,
		O  => xSYNC_USB);
--------------------------------------------------------------------------------		
	xBUF_SLRD : BUF 
	port map (
		I  => SLRD,
		O  => xSLRD);
--------------------------------------------------------------------------------		
	xBUF_SLOE : BUF 
	port map (
		I  => SLOE,
		O  => xSLOE);
--------------------------------------------------------------------------------		
	xBUF_RBUSY : BUF 
	port map (
		I  => RBUSY,
		O  => xRBUSY);
--------------------------------------------------------------------------------		
	xBUF_FIFOADR0 : BUF 
	port map (
		I  => FIFOADR(0),
		O  => xFIFOADR(0));
--------------------------------------------------------------------------------		
	xBUF_FIFOADR1 : BUF 
	port map (
		I  => FIFOADR(1),
		O  => xFIFOADR(1));	
--------------------------------------------------------------------------------
	process(xIFCLK, xRESET)
	variable delay : integer range 0 to 10;
	begin
		if xRESET = '0' then
			REFRESH		<= '0';
			PED_COL_ADDR <= (others=>'0');
			PED_ROW_ADDR <= (others=>'0');
			THRES0		<= (others=>'0');
			THRES1		<= (others=>'0');
			THRES2		<= (others=>'0');
			THRES3		<= (others=>'0');
			WBIAS			<= (others=>'0');
			TERM			<= (others=>'0');
			SGN			<= '0';
			EN_PED		<= '0';
			SYNC_USB		<= '0';
			SOFT_TRIG	<= '0';
			SLRD 			<= '1';
			SLOE 			<= '1';
			FIFOADR 		<= "10";
			TOGGLE 		<= '0';
			again 		<= "00";
			RBUSY 		<= '1';
			delay 		:= 0;	
			state       <= st1_WAIT;
		elsif rising_edge(xIFCLK) then
			SLOE 			<= '1';
			SLRD 			<= '1';			
			FIFOADR 		<= "10";
			TOGGLE 		<= '0';
			SOFT_TRIG	<= '0';
			REFRESH	 	<= '0';
			RBUSY 		<= '1';
--------------------------------------------------------------------------------				
			case	state is	
--------------------------------------------------------------------------------
				when st1_WAIT =>
					RBUSY <= '0';						 
					if xFLAGA = '1' then	
						RBUSY <= '1';
						if xWBUSY = '0' then		
							RBUSY <= '1';
							state <= st1_READ;
						end if;
					end if;		 
--------------------------------------------------------------------------------		
				when st1_READ =>
					FIFOADR <= "00";	
					TOGGLE <= '1';		
					if delay = 2 then
						delay := 0;
						state <= st2_READ;
					else
						delay := delay +1;
					end if;
--------------------------------------------------------------------------------					
				when st2_READ =>	
					FIFOADR <= "00";
					TOGGLE <= '1';
					SLOE <= '0';
					if delay = 2 then
						delay := 0;
						state <= st3_READ;
					else
						delay := delay +1;
					end if;				
--------------------------------------------------------------------------------						
				when st3_READ =>					
					FIFOADR <= "00";
					TOGGLE <= '1';
					SLOE <= '0';
					SLRD <= '0';
					dbuffer <= xUSB_DATA;
					if delay = 2 then
						delay := 0;
						state <= st4_READ;
					else
						delay := delay +1;
					end if;					
--------------------------------------------------------------------------------					   
				when st4_READ =>					
					FIFOADR <= "00";
					TOGGLE <= '1';
					SLOE <= '0';
					if delay = 2 then
						delay := 0;
						state <= st1_SAVE;
					else
						delay := delay +1;
					end if;				
--------------------------------------------------------------------------------	
				when st1_SAVE	=>
					FIFOADR <= "00";
					TOGGLE <= '1';	
--------------------------------------------------------------------------------						
					case again is 
						when "00" =>	
							again <="01";	
							Locmd <= dbuffer;
							state <= ENDDELAY;
--------------------------------------------------------------------------------	
						when "01" =>
							again <="00";	
							Hicmd <= dbuffer;	
							state <= st1_TARGET;
--------------------------------------------------------------------------------	
						when others =>				
							state <= st1_WAIT;
					end case;
--------------------------------------------------------------------------------	
				when st1_TARGET =>	
					RBUSY <= '0';	
					case Locmd(12 downto 0) is
--------------------------------------------------------------------------------	
-----------       Software People only care about this part :-p    -------------	
--------------------------------------------------------------------------------							
						when '0' & x"001" =>	--USE SYNC signal		
							SYNC_USB <= Hicmd(0); 	--LOCMD "XXX0-0000-0000-0001"
							state <= st1_WAIT;		--HICMD "XXXX-XXXX-XXXX-XXXD"
									
						when '0' & x"002" =>	--Software Trigger
							SOFT_TRIG <= '1';	 		--LOCMD "XXX0-0000-0000-0010"
							state <= st1_WAIT;		

						when '0' & x"003" => --Load data to THRES			
							if Hicmd(15 downto 14) = "00" then
								THRES0 <=				  	--LOCMD "XXX0-0000-0000-0011"
								Hicmd(15 downto 0); 		--HICMD "00XX-DDDD-DDDD-DDDD" 
							elsif Hicmd(15 downto 14) = "01" then
								THRES1 <=				  	--LOCMD "XXX0-0000-0000-0011"
								Hicmd(15 downto 0); 		--HICMD "01XX-DDDD-DDDD-DDDD" 
							elsif Hicmd(15 downto 14) = "10" then
								THRES2 <=				  	--LOCMD "XXX0-0000-0000-0011"
								Hicmd(15 downto 0); 		--HICMD "10XX-DDDD-DDDD-DDDD" 
							elsif Hicmd(15 downto 14) = "11" then
								THRES3 <=				  	--LOCMD "XXX0-0000-0000-0011"
								Hicmd(15 downto 0); 		--HICMD "11XX-DDDD-DDDD-DDDD" 
							end if;
							REFRESH	<= '1';
							state 	<= st1_WAIT;					
							
						when '0' & x"004" =>	--Load data to EN_PED
							EN_PED <= Hicmd(0); 		--LOCMD "XXX0-0000-0000-0100" 
															--HICMD "XXXX-XXXX-XXXX-XXXD"
							REFRESH	<= '1';
							state 	<= st1_WAIT;					
							
						when '0' & x"005" =>	--Load data to PED_ADDR
							PED_COL_ADDR      --LOCMD "XXX0-0000-0000-0101" 
								<= Hicmd(8 downto 4);
							PED_ROW_ADDR		--HICMD "XXXX-DDDD-DDDD-XXXX"
								<= Hicmd(11 downto 9); 	
							REFRESH	<= '1';
							state 	<= st1_WAIT;	

						when '0' & x"006" =>	--Load data to WBIAS
							WBIAS <=	Hicmd(11 downto 0);	--LOCMD "XXX0-0000-0000-0110"
							REFRESH	<= '1';					--HICMD "DDDD-DDDD-DDDD-DDDD" 
							state 	<= st1_WAIT;	

							when '0' & x"007" =>	--Load data to SGN
							SGN <= Hicmd(0);			  		--LOCMD "XXX0-0000-0000-0111"
							REFRESH	<= '1';						--HICMD "XXXX-XXXX-XXXX-XXXD"
							state 	<= st1_WAIT;	

							when '0' & x"008" =>	--Load data to TERM
							TERM <= Hicmd(2 downto 0);		--LOCMD "XXX0-0000-0000-1000" 
							REFRESH	<= '1';					--HICMD "XXXX-XXXX-XXXX-XDDD"	
							state 	<= st1_WAIT;	
--------------------------------------------------------------------------------	
-----------       Software People stop caring at this point  ^_^   -------------	
--------------------------------------------------------------------------------																			
						when others =>
							state <= st1_WAIT;
					end case;
--------------------------------------------------------------------------------	
				when ENDDELAY =>	
					FIFOADR <= "00";
					if delay = 2 then
						if xFLAGA = '1' then
							delay := 0;
							state <= st1_READ;
						else
							delay := 0;
							state <= st1_WAIT;
						end if;
					else
						delay := delay +1;
					end if;
--------------------------------------------------------------------------------						
				when others =>
					state <= st1_WAIT;
			end case;	  
		end if;
	end process;
--------------------------------------------------------------------------------	
end Behavioral;
